/**
 * Created by Tony on 2017/9/14.
 */

import mongoose from "mongoose";

let rpt_tpl_tree_mdl = mongoose.model("rpt_tpl_tree");

const SYSTEM_USER_ID = -100;

function validate(doc) {
    let rst = true;
    if (doc) {
        if (!(doc.hasOwnProperty("comilationId") && doc.hasOwnProperty("engineerId") && doc.hasOwnProperty("userId"))) {
            rst = false;
        }
    } else {
        rst = false;
    }
    return rst;
}

async function createNewTree(doc) {
    if (validate(doc)) {
        return await rpt_tpl_tree_mdl.create(doc)
    } else {
        return null;
    }
}

async function updateTree(comilationId, engineerId, userId, doc) {
    let filter = {"comilationId": comilationId, "engineerId": engineerId, "userId": userId};
    return await rpt_tpl_tree_mdl.update(filter, doc);
}

async function removeTree(comilationId, engineerId, userId, cb) {
    let filter = {"comilationId": comilationId, "engineerId": engineerId, "userId": userId};
    return await rpt_tpl_tree_mdl.findAndModify(filter, [], { $set: { "isDeleted": true } }, {'new':true}, cb);
}

async function removeTreePhycically(comilationId, engineerId, userId) {
    let filter = {"comilationId": comilationId, "engineerId": engineerId, "userId": userId};
    return await rpt_tpl_tree_mdl.remove(filter);
}

async function findTplTree(comilationId, engineerIds, userIds) {
    //let filter = {"comilationId": comilationId, "engineerId": engineerIds, "userId": userIds};
    let filter = {"comilationId": comilationId, "engineerId": engineerIds, "userId": userIds, "$or": [{"isDeleted": null}, {"isDeleted": false}]};
    if (engineerIds instanceof Array) {
        filter.engineerId = {$in: engineerIds};
    }
    if (userIds instanceof Array) {
        filter.userId = {$in: userIds};
    }
    return await rpt_tpl_tree_mdl.find(filter);
}

async function findTplTreeByOid(objectId) {
    if (objectId) {
        //let filter = {"_id": ObjectId(objectId)};
        let id = mongoose.Types.ObjectId(objectId);
        let filter = {_id: id};
        return await rpt_tpl_tree_mdl.findOne(filter);
    } else return null;
}


let expObj = {
    createNewTree:  createNewTree,
    updateTree:     updateTree,
    removeTree:     removeTree,
    removeTreePhycically: removeTreePhycically,
    findTplTree:    findTplTree,
    findTplTreeByOid: findTplTreeByOid
};

export {expObj as default};